""" Game fix for FINAL FANTASY IX
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Changes the proton argument from the launcher to the game
    """

    log('Applying fixes for FINAL FANTASY IX')

    # Fix crackling audio
    util.set_environment('PULSE_LATENCY_MSEC', '60')

    # Replace launcher with game exe in proton arguments
    util.replace_command('FF9_Launcher.exe', 'x64/FF9.exe')



""" Game fix for Order of Battle: World War II
Still missing intro video codecs
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Install corefonts
    """

    log('Applying fixes for Order of Battle: World War II')

    # https://github.com/ValveSoftware/Proton/issues/639
    util.protontricks('corefonts')



""" Game fix for Grand Theft Auto V
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Install corefonts
    """

    log('Applying fixes for Grand Theft Auto V')

    # https://github.com/ValveSoftware/Proton/issues/37
    util.protontricks('corefonts')



""" Game fix for SUGURI 2
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Installs d3dxof
    """

    log('Applying fixes for SUGURI 2')

    # https://github.com/ValveSoftware/Proton/issues/970#issuecomment-420421289
    util.protontricks('d3dxof')



""" Game fix for BioShock 2 Remastered
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Disable ESYNC, disable intro's
    """

    log('Applying fixes for BioShock 2 Remastered')

    # After loading the game, or a save file, a key needs to be pressed
    # to continue. That screen does not respond to keyboard or mouse,
    # so there is no way to continue. -nointro disables that screen
    # (but also the intro's at the start of the game).
    util.append_argument('-nointro')

    # ESYNC causes texture problems and frequent hangs.
    util.set_environment('PROTON_NO_ESYNC', '1')



""" Game fix for Doom 2016
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Install vcrun2015
    """

    log('Applying fixes for Doom 2016')

    # https://github.com/ValveSoftware/Proton/issues/788#issuecomment-416651267
    util.protontricks('vcrun2015')



""" Game fix for Titan Quest Anniversary Edition(475150)
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Set OS to Windows XP to pass the black menu screen
    """

    log('Applying Titan Quest Anniversary Edition Game Fixes')

    util.protontricks('winxp')



""" Game fix for Killer is Dead at Launch
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Installs dotnet45 directx9 vcruntime2010 and xact_june2018 and sets to winxp
    """

    log('Applying fixes for Killer is Dead at Launch')

    # https://github.com/ValveSoftware/Proton/issues/1387#issuecomment-428059647
    util.protontricks('winxp')
    util.protontricks('dotnet45')
    util.protontricks('directx9')
    util.protontricks('vcruntime2010')
    util.protontricks('xact_june2018')



""" Splash screen for protonfixes using cefpython
"""
import os
import time
import subprocess
from multiprocessing import Process
from contextlib import contextmanager
from .logger import log


def browser(cef, url):
    """ Starts a cef browser in the middle of the screen with url
    """

    # Keeps the splash from displaying on short tasks
    log.info('Delaying splash for 2 seconds')
    time.sleep(2)
    log.debug('Starting splash screen')

    settings = {
        'background_color': 0xff000000,
        'cache_path': '',
        'context_menu': {'enabled': False},
        'debug': False,
        'command_line_args_disabled': False,
    }
    switches = {
        'disable-gpu': '',
        'disable-gpu-compositing': '',
    }

    cef.Initialize(settings, switches)

    win_info = cef.WindowInfo()
    win_info.SetAsChild(0, coordinates(600, 360))

    cef.CreateBrowser(url=url, window_info=win_info, window_title='splash')
    cef.MessageLoop()
    cef.Shutdown()

def coordinates(width, height):
    """ Returns coordinates [x1, y1, x2, y2] for a centered box of width, height
    """

    with open('/sys/class/graphics/fb0/virtual_size', 'r') as res:
        screenx, screeny = map(int, res.read().strip('\n').split(','))

    posx = (screenx/2) - (width/2)
    posy = (screeny/2) - (height/2)

    return [posx, posy, posx+width, posy+height]


def sys_zenity_path():
    """ Returns the path of zenity if found in system $PATH
    """

    steampath = os.environ['PATH'].split(':')
    syspath = [x for x in steampath if 'steam-runtime' not in x]
    for path in syspath:
        zenity_path = os.path.join(path, 'zenity')
        if os.path.exists(zenity_path) and os.access(zenity_path, os.X_OK):
            return zenity_path
    return False


@contextmanager
def zenity_splash():
    """ Runs the zenity process until context is returned
    """

    log.debug('Starting zenity splash screen')

    zenity_bin = sys_zenity_path()
    if not zenity_bin:
        return

    zenity_cmd = ' '.join([
        'sleep 2;',
        zenity_bin,
        '--progress',
        '--pulsate',
        '--no-cancel',
        '--auto-close',
        '--text',
        '"ProtonFixes is running a task, please wait..."',
        ])

    # it would be better to use multiprocessing and time.sleep(2) here,
    # but zenity forks and won't quit when the subprocess is killed,
    # hence, using shell=True and 'sleep 2;'
    zenity = subprocess.Popen(zenity_cmd,
                              stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE,
                              shell=True,
                             )

    yield
    log.debug('Terminating zenity splash screen')
    zenity.kill()


@contextmanager
def cef_splash(cef, page='index.html'):
    """ Runs the browser process until the context is returned
    """

    log.debug('Starting CEF splash screen')
    data_dir = os.path.join(os.path.dirname(__file__), '..', 'static')
    url = 'file://' + os.path.join(data_dir, page)
    cef_proc = Process(target=browser, args=(cef, url))
    cef_proc.start()
    yield
    log.debug('Terminating CEF splash screen')
    cef_proc.terminate()


@contextmanager
def splash():
    """ Wrapper logic to choose the zenity or cef splash screen
    """

    log.debug('Starting splash screen')

    if 'SteamTenfoot' in os.environ:
        log.debug('Running in Big Picture mode')
        try:
            from cefpython3 import cefpython as cef
            log.debug('Using cefpython splash screen')
            with cef_splash(cef):
                yield
                return
        except ImportError:
            log.warn('Optional dependency cefpython3 not found')

    if sys_zenity_path():
        log.debug('Using zenity splash screen')
        with zenity_splash():
            yield
            return
    else:
        log.warn('Optional dependency zenity not found')

    log.warn('No splash dependencies found, running without splash screen')
    yield
    return



""" Game fix for You Need a Budget 4
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Installs corefonts
    """

    log('Applying fixes for You Need a Budget 4')

    # https://github.com/ValveSoftware/Proton/issues/7
    util.protontricks('corefonts')



""" Run some tests and generate warnings for proton configuration issues
"""

from .logger import log

ESYNC_WARNING = ''' File descriptor limit is low
This can cause issues with ESYNC
For more details see:
https://github.com/zfigura/wine/blob/esync/README.esync
'''

# https://www.reddit.com/r/SteamPlay/comments/9kqisk/tip_for_those_using_proton_no_esync1/
#/proc/sys/fs/file-max
def esync_file_limits():
    with open('/proc/sys/fs/file-max') as fsmax:
        max_files = fsmax.readline()
        if int(max_files) < 8192:
            log.warn(ESYNC_WARNING)

esync_file_limits()



""" Game fix for Chronophantasma Extend
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Uses win32 prefix, installs quartz devenum wmp9 xact
    """

    log('Applying fixes for Chronophantasma Extend')

    # Winetrick wmp9 requires 32bit prefix
    util.use_win32_prefix()

    # https://github.com/ValveSoftware/Proton/issues/703#issuecomment-416075961
    util.protontricks('devenum')
    util.protontricks('quartz')
    util.protontricks('wmp9')
    util.protontricks('xact')



""" Utilities to make gamefixes easier
"""

import configparser
import os
import sys
import shutil
import signal
import subprocess
from .logger import log
try:
    import __main__ as protonmain
except ImportError:
    log.warn('Unable to hook into Proton main script environment')

# pylint: disable=unreachable

def which(appname):
    """ Returns the full path of an executable in $PATH
    """

    for path in os.environ['PATH'].split(os.pathsep):
        fullpath = os.path.join(path, appname)
        if os.path.exists(fullpath) and os.access(fullpath, os.X_OK):
            return fullpath
    log.warn(str(appname) + 'not found in $PATH')
    return None


def protondir():
    """ Returns the path to proton
    """

    proton_dir = os.path.dirname(sys.argv[0])
    return proton_dir


def protonprefix():
    """ Returns the wineprefix used by proton
    """

    return os.path.join(
        os.environ['STEAM_COMPAT_DATA_PATH'],
        'pfx/')


def _killhanging():
    """ Kills processes that hang when installing winetricks
    """

    # avoiding an external library as proc should be available on linux
    log.debug('Killing hanging wine processes')
    pids = [pid for pid in os.listdir('/proc') if pid.isdigit()]
    badexes = ['mscorsvw.exe']
    for pid in pids:
        try:
            with open(os.path.join('/proc', pid, 'cmdline'), 'rb') as proc_cmd:
                cmdline = proc_cmd.read()
                for exe in badexes:
                    if exe in cmdline.decode():
                        os.kill(int(pid), signal.SIGKILL)
        except IOError:
            continue

def _del_syswow64():
    """ Deletes the syswow64 folder
    """

    try:
        shutil.rmtree(os.path.join(protonprefix(), 'drive_c/windows/syswow64'))
    except FileNotFoundError:
        log.warn('The syswow64 folder was not found')

def _mk_syswow64():
    """ Makes the syswow64 folder
    """

    try:
        os.makedirs(os.path.join(protonprefix(), 'drive_c/windows/syswow64'))
    except FileExistsError:
        log.warn('The syswor64 folder already exists')


def checkinstalled(verb):
    """ Returns True if the winetricks verb is found in the winetricks log
    """

    if not isinstance(verb, str):
        return False

    log.info('Checking if winetricks ' + verb + ' is installed')
    winetricks_log = os.path.join(protonprefix(), 'winetricks.log')
    try:
        with open(winetricks_log, 'r') as tricklog:
            if verb in [x.strip() for x in tricklog.readlines()]:
                return True
    except OSError:
        return False
    return False


def protontricks(verb):
    """ Runs winetricks if available
    """

    if not checkinstalled(verb):
        log.info('Installing winetricks ' + verb)
        env = dict(os.environ)
        env['WINEPREFIX'] = protonprefix()
        env['WINESERVER'] = os.path.join(protondir(), 'dist/bin/wineserver')
        env['WINETRICKS_LATEST_VERSION_CHECK'] = 'disabled'

        winetricks_bin = which('winetricks')
        winetricks_cmd = [winetricks_bin, '--unattended', '--force'] + verb.split(' ')
        wineserver_bin = env['WINESERVER']

        if winetricks_bin is None:
            log.warn('No winetricks was found in $PATH')

        if winetricks_bin is not None:

            log.debug('Using winetricks command: ' + str(winetricks_cmd))
            # winetricks relies entirely on the existence of syswow64 to determine
            # if the prefix is 64 bit, while proton fails to run without it
            log.debug('Deleting syswow64')
            if 'win32' in protonprefix():
                _del_syswow64()

            # make sure proton waits for winetricks to finish
            for idx, arg in enumerate(sys.argv):
                if 'waitforexitandrun' not in arg:
                    sys.argv[idx] = arg.replace('run', 'waitforexitandrun')
                    log.debug(str(sys.argv))

            log.info('Using winetricks verb ' + verb)
            process = subprocess.Popen(winetricks_cmd, env=env)
            process.wait()
            _killhanging()
            subprocess.Popen([wineserver_bin, '-k'], env=env)
            log.info('Winetricks complete')
            return True

            # restore syswow64 so proton doesn't crash
            log.info('Restoring syswow64 folder')
            if 'win32' in protonprefix():
                _mk_syswow64()
    return False


def win32_prefix_exists():
    """ Returns True if there is a _win32 path available
    """

    prefix32 = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32/pfx'
    if os.path.exists(prefix32):
        log.debug('Found win32 prefix')
        return True
    log.debug('No win32 prefix found')
    return False


def use_win32_prefix():
    """ Sets variables in the main proton script to use the _win32 prefix
    """
    if not win32_prefix_exists():
        make_win32_prefix()

    data_path = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32'
    prefix32 = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32/pfx/'

    protonmain.env['STEAM_COMPAT_DATA_PATH'] = data_path
    protonmain.env['WINEPREFIX'] = prefix32
    protonmain.env['WINEDLLPATH'] = os.path.join(protondir(), 'dist/lib/wine')
    protonmain.env['WINEARCH'] = 'win32'
    protonmain.prefix = prefix32

    os.environ['STEAM_COMPAT_DATA_PATH'] = data_path
    os.environ['WINEPREFIX'] = prefix32
    os.environ['WINEDLLPATH'] = os.path.join(protondir(), 'dist/lib/wine')
    os.environ['WINEARCH'] = 'win32'
    log.debug('Updated environment variables for win32 prefix')
    # make sure steam doesn't crash when missing syswow
    try:
        os.makedirs(os.path.join(prefix32, 'drive_c/windows/syswow64'))
    except FileExistsError:
        pass


def make_win32_prefix():
    """ Creates a win32 prefix
    """

    env = dict(os.environ)
    log.info('Bootstrapping win32 prefix')

    prefix32 = os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32/pfx'
    try:
        os.makedirs(prefix32)

        env['WINEARCH'] = 'win32'
        env['WINEPREFIX'] = prefix32
        server = subprocess.Popen([which('wineserver'), '-f'], env=env)
        subprocess.Popen([which('wine'), 'wineboot', '--init'], env=env)
        server.wait()

        os.makedirs(os.path.join(prefix32, 'drive_c/windows/syswow64'))
        os.symlink(
            os.path.join(prefix32, 'drive_c/Program Files'),
            os.path.join(prefix32, 'drive_c/Program Files (x86)'))

        log.info('Initialized win32 prefix')

    except OSError:
        log.warn('Directory for win32 prefix already exists')


def replace_command(orig_str, repl_str):
    """ Make a commandline replacement in sys.argv
    """

    log.info('Changing ' + orig_str + ' to ' + repl_str)
    for idx, arg in enumerate(sys.argv):
        if orig_str in arg:
            sys.argv[idx] = arg.replace(orig_str, repl_str)

def append_argument(argument):
    """ Append an argument to sys.argv
    """

    log.info('Adding argument ' + argument)
    sys.argv.append(argument)
    log.debug('New commandline: ' + str(sys.argv))

def set_environment(envvar, value):
    """ Add or override an environment value
    """

    log.info('Adding env: ' + envvar + '=' + value)
    os.environ[envvar] = value
    protonmain.env[envvar] = value

def get_game_install_path():
    """ Game installation path
    """

    log.debug('Detected path to game: ' + os.environ['PWD'])
    # only for `waitforexitandrun` command
    return os.environ['PWD']

def winedll_override(dll, dtype):
    """ Add WINE dll override
    """

    log.info('Overriding ' + dll + '.dll = ' + dtype)
    protonmain.dlloverrides[dll] = dtype

def disable_nvapi():
    """ Disable WINE nv* dlls
    """

    log.info('Disabling NvAPI')
    winedll_override('nvapi', '')
    winedll_override('nvapi64', '')
    winedll_override('nvcuda', '')
    winedll_override('nvcuvid', '')
    winedll_override('nvencodeapi', '')
    winedll_override('nvencodeapi64', '')

def disable_d3d10():
    """ Disable WINE d3d10* dlls
    """

    log.info('Disabling d3d10')
    winedll_override('d3d10', '')
    winedll_override('d3d10_1', '')
    winedll_override('d3d10core', '')

def disable_dxvk():  # pylint: disable=missing-docstring
    set_environment('PROTON_USE_WINED3D11', '1')

def disable_esync():  # pylint: disable=missing-docstring
    set_environment('PROTON_NO_ESYNC', '1')

def disable_d3d11():  # pylint: disable=missing-docstring
    set_environment('PROTON_NO_D3D11', '1')


def create_dosbox_conf(conf_file, conf_dict):
    """Create DOSBox configuration file.

    DOSBox accepts multiple configuration files passed with -conf
    option;, each subsequent one overwrites settings defined in
    previous files.
    """
    if os.access(conf_file, os.F_OK):
        return
    conf = configparser.ConfigParser()
    conf.read_dict(conf_dict)
    with open(conf_file, 'w') as file:
        conf.write(file)



""" Tests for the util.py helpers
"""

import os
import sys
import glob
import time
import shutil
import subprocess
import unittest
from protonfixes import util


class Quiet:
    def __enter__(self):
        self.stdout, self.stderr = sys.stdout, sys.stderr
        self.null = open(os.devnull, 'w')
        sys.stdout = sys.stderr = self.null

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.null.close()
        sys.stdout, sys.stderr = self.stdout, self.stderr

def setup_testenv(gameid):
    """ Setup the test environment variables
    """

    sys.argv[0] = glob.glob(os.path.expanduser('~/.local/share/Steam/steamapps/common/Proton*/proton'))[0]

    test_env = {
        'STEAM_COMPAT_DATA_PATH': '/tmp/protonfixes_test/' + str(gameid),
        'SteamUser': 'steamuser',
        'ENABLE_VK_LAYER_VALVE_steam_overlay_1': '1',
        'WINEDEBUG': '-all',
        }

    test_env['STEAM_COMPAT_CLIENT_INSTALL_PATH'] = os.path.expanduser('~/.local/share/Steam')
    test_env['STEAM_RUNTIME'] = os.path.expanduser('~/.local/share/Steam/ubuntu12_32/steam-runtime')

    os.environ.update(test_env)
    return test_env


def setup_testpfx(gameid):
    """ Create a test prefix
    """

    test_env = setup_testenv(gameid)
    os.environ.update(test_env)
    prefix = test_env['STEAM_COMPAT_DATA_PATH']
    try:
        shutil.rmtree(prefix)
    except FileNotFoundError:
        pass
    os.makedirs(prefix)
    proton_bin = glob.glob(os.path.join(
        test_env['STEAM_COMPAT_CLIENT_INSTALL_PATH'], 'steamapps/common/Proton*/proton'))[0]

    with open(os.devnull, 'w') as devnull:
        subprocess.run(
            [proton_bin, 'getnativepath', 'C:\\windows\\notepad.exe'],
            stdout=devnull, stderr=devnull)

    return prefix


def tearDownModule():
    """ Remove the protonfixes_test dir from /tmp
    """
    try:
        shutil.rmtree('/tmp/protonfixes_test/')
    except OSError:
        time.sleep(1)
        shutil.rmtree('/tmp/protonfixes_test/')

@unittest.skipUnless(os.name == 'posix', 'Not testing on windows')
class TestWhich(unittest.TestCase):
    """ util.py which tests
    """
    def test_return(self):
        """ Should return exactly like shutil.which
        """

        self.assertEqual(shutil.which('hostname'), util.which('hostname'))

    def test_executable(self):
        """ The returned path should be executable
        """

        true_bin = util.which('true')
        return_code = subprocess.call(true_bin, env={})

        self.assertEqual(return_code, 0)


@unittest.skipUnless(os.name == 'posix', 'Not testing on windows')
class TestProtonDir(unittest.TestCase):
    """ util.py protondir tests
    """
    @classmethod
    def setUpClass(cls):
        setup_testenv(1000)

    def test_is_dir(self):
        """ Check if a directory is returned
        """

        proton_dir = util.protondir()
        self.assertTrue(os.path.isdir(proton_dir))

    def test_return(self):
        """ Check if proton files are in directory
        """

        proton_files = os.listdir(util.protondir())
        self.assertTrue('proton' in proton_files)
        self.assertTrue('toolmanifest.vdf' in proton_files)
        self.assertTrue('proton_dist.tar.gz' in proton_files)
        self.assertTrue('version' in proton_files)
        self.assertTrue('dist' in proton_files)

@unittest.skipUnless(os.name == 'posix', 'Not testing on windows')
class TestProtonPrefix(unittest.TestCase):
    """ util.py protonprefix tests
    """

    @classmethod
    def setUpClass(cls):
        setup_testpfx(1001)

    def test_is_dir(self):
        """ Check if a directory is returned
        """

        prefix = util.protonprefix()
        self.assertTrue(os.path.isdir(prefix))

    def test_prefix_valid(self):
        """ Check if prefix contains required files
        """

        prefix_files = os.listdir(util.protonprefix())
        self.assertTrue('dosdevices' in prefix_files)
        self.assertTrue('drive_c' in prefix_files)
        self.assertTrue('userdef.reg' in prefix_files)
        self.assertTrue('user.reg' in prefix_files)


@unittest.skipUnless(os.name == 'posix', 'Not testing on windows')
class TestProtonTricks(unittest.TestCase):
    """ util.py protontricks tests
    """

    @classmethod
    def setUpClass(cls):
        setup_testpfx(1002)

    def test_installs(self):
        """ Check that protrontricks installs a verb
        """

        with Quiet():
            result = util.protontricks('good')
        self.assertTrue(result)

        prefix = os.path.join(os.environ['STEAM_COMPAT_DATA_PATH'], 'pfx')

        with open(os.path.join(prefix, 'winetricks.log')) as log:
            self.assertTrue('good' in log.read())


@unittest.skipUnless(os.name == 'posix', 'Not testing on windows')
class TestCheckInstalled(unittest.TestCase):
    """ util.py checkinstalled tests
    """

    @classmethod
    def setUpClass(cls):
        setup_testenv(1003)
        logdir = os.path.join(os.environ['STEAM_COMPAT_DATA_PATH'], 'pfx')
        os.makedirs(logdir)
        with open(os.path.join(logdir, 'winetricks.log'), 'w') as log:
            log.write('good')

    def test_return_installed(self):
        """ Check True is returned if verb is in winetricks.log
        """

        with Quiet():
            checkinstalled = util.checkinstalled('good')
        self.assertTrue(checkinstalled)

    def test_return_uninstalled(self):
        """ Check False is returned if verb is not in winetricks.log
        """

        with Quiet():
            checkinstalled = util.checkinstalled('bad')
        self.assertFalse(checkinstalled)

    def test_bad_input(self):
        """ Check False is returned if verb is not a string
        """
        with Quiet():
            zero = util.checkinstalled(0)
            mtlist = util.checkinstalled([])

        self.assertFalse(zero)
        self.assertFalse(mtlist)


@unittest.skipUnless(os.name == 'posix', 'Not testing on windows')
class TestMakeWin32Prefix(unittest.TestCase):
    """ util.py make_win32_prefix tests
    """

    @classmethod
    def setUpClass(cls):
        setup_testenv(1004)

    def test_creates_prefix(self):
        """ Check that a valid win32 prefix is created
        """
        with Quiet():
            util.make_win32_prefix()
        prefix_files = os.listdir(os.environ['STEAM_COMPAT_DATA_PATH'] + '_win32/pfx')
        self.assertTrue('dosdevices' in prefix_files)
        self.assertTrue('drive_c' in prefix_files)
        self.assertTrue('userdef.reg' in prefix_files)
        self.assertTrue('user.reg' in prefix_files)



""" Game fix for Oddworld: Abe's Oddysee
TODO: Fix steam controller input, it is stuck in lizard mode without overlay
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Adds the -interline argument to the game
    """

    log('Applying fixes for Oddworld: Abe\'s Oddysee')

    # Adding -interline fixes slow videos but adds scanlines
    util.append_argument('-interline')



""" Game fix for STAR WARS Jedi Knight II - Jedi Outcast
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Set Mesa env vars
    """

    log('Applying fixes for STAR WARS Jedi Knight II - Jedi Outcast')

    # https://github.com/ValveSoftware/Proton/pull/1423/commits/1a1d25c7d95691e37c94aea4e5f94e0c917aba6f
    util.set_environment('MESA_EXTENSION_MAX_YEAR', '2003')
    util.set_environment('__GL_ExtensionStringVersion', '17700')



""" Prints debug info if the environment variable DEBUG is 1
"""

import os
import sys
import shutil
# pylint: disable=E0611
from __main__ import CURRENT_PREFIX_VERSION, basedir
from .logger import log

os.environ['DEBUG'] = '1'

LINE = '---------------------------------------'
log.debug('---- begin protontricks debug info ----')
log.debug('Proton Python Version:')
log.debug(sys.executable)
log.debug(sys.version)
log.debug(LINE)
log.debug('System Python Version:')
try:
    log.debug(shutil.which(os.readlink(shutil.which('python'))))
except: #pylint: disable=W0702
    log.debug(shutil.which('python'))
log.debug(LINE)

log.debug('Proton Version:')
log.debug(CURRENT_PREFIX_VERSION)
log.debug(LINE)

log.debug('Proton Directory:')
log.debug(basedir)
log.debug(LINE)

IGNOREVARS = [
    'SteamUser',
    'OLDPWD',
    'SDL_GAMECONTROLLERCONFIG',
    'SteamAppUser',
    'SDL_GAMECONTROLLER_IGNORE_DEVICES',
    'LS_COLORS',
]

log.debug('Environment Variables:')
for key, value in os.environ.items():
    if key not in IGNOREVARS:
        log.debug(key + '=' + value)
log.debug(LINE)

log.debug('Command Line:')
log.debug(sys.argv)
log.debug('----- end protontricks debug info -----')



""" Game fix for Age of Mythology: Extended Edition
Source: https://github.com/JamesHealdUK/protonfixes/blob/master/fixes/266840.sh
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Changes the proton argument from the launcher to the game
    """

    log('Applying fixes for Age of Mythology: Extended Edition')

    # Replace launcher with game exe in proton arguments
    util.replace_command('Launcher.exe', 'aomx.exe')



""" Game fix for Battlefield: Bad Company 2
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Install corefonts, dx9. Set to win7 and override msdmo
    """

    log('Applying fixes for Battlefield: Bad Company 2')

    # https://github.com/ValveSoftware/Proton/issues/200#issuecomment-415905979
    util.protontricks('win7')
    util.protontricks('corefonts')
    util.protontricks('directx9')
    util.set_environment('WINEDLLOVERRIDES', 'msdmo=b')



""" Game fix for Oddworld: Munch's Oddysee
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Uses win32 prefix, installs devenum, quartz, wmp9
    """

    log('Applying fixes for Oddworld: Munch\'s Oddysee')

    # Winetrick wmp9 requires 32bit prefix
    util.use_win32_prefix()

    # https://appdb.winehq.org/objectManager.php?sClass=version&iId=34367
    util.protontricks('devenum')
    util.protontricks('quartz')
    util.protontricks('wmp9')



""" Game fix for The Evil Within(268050)
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Changes the proton argument from the launcher to the game
    """

    log('Applying The Evil Within Fixes')

    util.protontricks('xact')
    util.protontricks('win7')

    util.set_environment('PULSE_LATENCY_MSEC', '60')



""" Game fix for Tomb Raider I
"""
# pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Enable Glide emulation in dosbox config """

    log('Applying fixes for Tomb Raider I')

    conf_dict = {'glide': {'glide': 'emu'}}
    util.create_dosbox_conf('glide_fix.conf', conf_dict)
    util.append_argument('-conf')
    util.append_argument('glide_fix.conf')



""" Game fix for Civilization 4 (Beyond the Sword)
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Install msxml3
    """

    log('Applying fixes for Civilization 4 (Beyond the Sword)')

    # https://github.com/ValveSoftware/Proton/issues/179#issuecomment-415593087
    util.protontricks('msxml3')
    util.protontricks('msxml4')



""" Gets the game id and applies a fix if found
"""

from __future__ import print_function
import os
import re
import sys
from importlib import import_module
from protonfixes.splash import splash
from .logger import log

def game_id():
    """ Trys to return the game id from environment variables
    """

    if 'SteamAppId' in os.environ:
        return os.environ['SteamAppId']
    if 'SteamGameId' in os.environ:
        return os.environ['SteamGameId']
    if 'STEAM_COMPAT_DATA_PATH' in os.environ:
        return re.findall(r'\d+', os.environ['STEAM_COMPAT_DATA_PATH'])[-1]

    log.crit('Game ID not found in environment variables')
    return None


def run_fix(gameid):
    """ Loads a gamefix module by it's gameid
    """

    if gameid is not None:
        localpath = os.path.expanduser('~/.config/protonfixes/localfixes')
        if os.path.isfile(os.path.join(localpath, gameid + '.py')):
            open(os.path.join(localpath, '__init__.py'), 'a').close()
            sys.path.append(os.path.expanduser('~/.config/protonfixes'))
            try:
                game_module = import_module('localfixes.' + gameid)
                log.info('Using local protonfix for gameid ' + gameid)
                game_module.main()
            except ImportError:
                log.info('No local protonfix found for gameid ' + gameid)
        else:
            try:
                game_module = import_module('protonfixes.gamefixes.' + gameid)
                log.info('Using protonfix for gameid ' + gameid)
                game_module.main()
            except ImportError:
                log.info('No protonfix found for gameid ' + gameid)


def main():
    """ Runs the gamefix, with splash if zenity or cefpython3 is available
    """

    if 'iscriptevaluator.exe' in sys.argv[2]:
        log.debug('Not running protonfixes for iscriptevaluator.exe')
        return

    if 'getcompatpath' in sys.argv[1]:
        log.debug('Not running protonfixes for getcompatpath')
        return

    if 'getnativepath' in sys.argv[1]:
        log.debug('Not running protonfixes for getnativepath')
        return

    log.info('Running protonfixes')
    with splash():
        run_fix(game_id())



""" Game fix for Styx: Master of Shadows
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Install dotnet40
    Works, but gives a popup twice at the beginning of launch:
    'Unable to find a version of the runtime to run this
    application. (OK)
    """

    log('Applying fixes for Styx: Master of Shadows')

    # https://github.com/ValveSoftware/Proton/issues/810
    # https://steamcommunity.com/app/242640/discussions/0/620700960990638817/
    util.protontricks('xact')
    util.protontricks('dotnet40')



""" Game fix for Age Of Empire 3: Complete Collection
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Uses win32 prefix, installs corefonts, dsound, l3codecx quarts mfc42 d3dx9 and dotnet20
    """

    log('Applying fixes for Age Of Empire 3: Complete Collection')

    # Winetrick wmp9 requires 32bit prefix
    util.use_win32_prefix()

    # https://github.com/ValveSoftware/Proton/issues/17#issuecomment-415977510
    util.protontricks('corefonts')
    util.protontricks('dsound')
    util.protontricks('l3codecx')
    util.protontricks('quartz')
    util.protontricks('mfc42')
    util.protontricks('d3dx9')
    util.protontricks('dotnet20')



""" Game fix for EVE Online
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Set to winxp
    """

    log('Applying fixes for EVE Online')

    # https://github.com/ValveSoftware/Proton/issues/1223#issue-356628050
    util.protontricks('winxp')



""" Install the protonfixes package
"""
import os
import glob
from setuptools import setup, find_packages

with open('README.md', 'r') as f:
    long_description = f.read()

data_files = []
bin_dir = os.path.join(os.path.dirname(__file__), 'static')
for root, dirs, files in os.walk(bin_dir):
    root_files = [os.path.join(root, i) for i in files]
    data_files.append((root, root_files))

print(data_files)

setup(
    name='protonfixes',
    version='1.0.12',
    description='Python module for applying fixes at runtime for games not supported by Steam Proton',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/simons-public/protonfixes',
    author='Chris Simons',
    author_email='chris@simonsmail.net',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Games/Entertainment',
        'Operating System :: POSIX :: Linux',
        'License :: OSI Approved :: BSD License',
        ],
    keywords='proton steam winetricks protonfixes',
    packages=find_packages(),
    zip_safe=False,
    data_files = data_files,
    )



""" Game fix for Forts
"""
#pylint: disable=C0103

from protonfixes import util
from protonfixes.logger import log

def main():
    """ Uses winetricks to install the ole32 verb
    """

    log('Applying fixes for Forts')

    util.protontricks('ole32')



""" Simple logging to stdout the same way Proton does
"""

import os
import sys

class Log():
    """Log to stderr for steam dumps
    """

    def __init__(self):
        self.pfx = 'ProtonFixes[' + str(os.getpid()) + '] '
        self.colors = {
            'RESET': '\u001b[0m',
            'INFO': '\u001b[34m',
            'WARN': '\u001b[33m',
            'CRIT': '\u001b[31m',
            'DEBUG': '\u001b[35m'
        }

    def __call__(self, msg):
        """ Allows the Log instance to be called directly
        """

        self.log(msg)

    def log(self, msg='', level='INFO'):
        """ Prints the log message to stdout the same way as Proton
        """

        pfx = self.pfx + level + ': '
        color = self.colors[level]
        reset = self.colors['RESET']
        fulltext = color + pfx + str(msg) + reset + os.linesep
        sys.stderr.write(fulltext)
        sys.stderr.flush()
        with open('/tmp/test', 'a', 1) as testfile:
            testfile.write(fulltext)


    def info(self, msg):
        """ Wrapper for printing info messages
        """

        self.log(msg, 'INFO')

    def warn(self, msg):
        """ Wrapper for printing warning messages
        """

        self.log(msg, 'WARN')

    def crit(self, msg):
        """ Wrapper for printing critical messages
        """

        self.log(msg, 'CRIT')

    def debug(self, msg):
        """ Wrapper for printing debug messages
        """

        if 'DEBUG' in os.environ:
            self.log(msg, 'DEBUG')


log = Log() #pylint: disable=C0103
